c
c   Program updowneq (UDQ)
c
c  $Id: updowneq.f,v 1.5 2009/08/11 06:35:21 agnew Exp agnew $
c  $Log: updowneq.f,v $
c  Revision 1.5  2009/08/11 06:35:21  agnew
c  Added flatten, float, altscale commands
c
c  Revision 1.4  2009/08/11 04:38:35  agnew
c  Fred Klein bug fixes, modification to read ANSS format as well as
c  blank-delimited format, longitudes reset to +-180
c
c  Revision 1.3  2009/03/04 19:52:46  agnew
c  longitudes moved to 0-360, seconds can be float
c
c  Revision 1.2  2009/03/02 05:28:25  agnew
c  added shape and color options
c
c  Revision 1.1  2009/02/25 00:20:58  agnew
c  Initial revision
c
c
c  Reads from seismicity listing (year month day hour min sec lat long dep mag)
c   and writes out the body of a .kml file in which the earthquakes are
c   represented by icons that fade away with time.
c
      parameter (inmx=500)
      parameter (maxeq=150000)
      parameter (maxspl=50)
      double precision time,start,finish,splpts,vanish
c
c common block for earthquake information:
c
      common /eqs/rlat(maxeq),rlong(maxeq),dep(maxeq),time(maxeq),
     1            rmag(maxeq),neq
c
c common block for display information
c
      common /display/vanish(maxeq),splpts(maxspl),start,finish,
     1 fade,nsplit,nspp
c
c common block for scaling depth to elevation
c
      common /depscl/ascale,zerlev
c
c
c common blocks of scanrp for handling commands:
c
      character*80 input(inmx)
      common /store/ input
      common /ndict/ iecho,nin
c
c
c Read commands; completed when reads  exec  command
c
      call scanrp
c
c Read in data
c
      call rdquake
c
c Determine split points for folders
c
      call dosplit
c
c Write out KML info
c
      call kmlmak
c
      stop
      end
c=======================================================================
      subroutine rdquake
c
c$$$$ calls getarr,getchr,getone,juldat,toymd
c
c  Reads in earthquake catalog information, keeping only events that fall within
c time, space, and magnitude limits. Catalog is assumed to have the format
c   year month day hour min sec lat long depth magnitude
c where the time information is integer, the lat and long are decimal
c degrees (East positive) and the depth in km
c  The field separator may be any of a space  , a slash /, or a colon :
c
      parameter (maxeq=150000)
      parameter (maxspl=50)
      double precision df,start,finish,ttime,time,splpts,vanish
      character*80 name,inline
      dimension rit(6),it(6),itm(3),geol(4)
c
c common block for earthquake information
c
      common /eqs/rlat(maxeq),rlong(maxeq),dep(maxeq),time(maxeq),
     1            rmag(maxeq),neq
c
c common block for display information
c
      common /display/vanish(maxeq),splpts(maxspl),start,finish,
     1 fade,nsplit,nspp
c
c common block for scaling depth to elevation
c
      common /depscl/ascale,zerlev
c
      data lui/11/
      neq=1
c
c get geographic limits
c
      call getarr('area',geol,4,nf)
c apparently GE 5.0.1 cannot handle longitudes above 180
c      if(geol(3).lt.0) geol(3)=geol(3)+360.
c      if(geol(4).lt.0) geol(4)=geol(4)+360.
      if(geol(3).gt.180.) geol(3)=geol(3)-360.
      if(geol(4).gt.180.) geol(4)=geol(4)-360.
c
c get temporal limits--which are converted to Julian day plus fraction of day
c
      call getarr('star',rit,6,nt)
      do i=1,nt
        it(i)=rit(i)
      enddo
      if(nt.ge.5) df=(it(nt-2)+(it(nt-1)+it(nt)/60.)/60.)/24.
      if(nt.eq.5) then
        call toymd(it,itm)
        jd=juldat(itm)
      endif
      if(nt.eq.6) jd=juldat(it)
      start=jd+df
      call getarr('fini',rit,6,nt)
      do i=1,nt
        it(i)=rit(i)
      enddo
      if(nt.ge.5) df=(it(nt-2)+(it(nt-1)+it(nt)/60.)/60.)/24.
      if(nt.eq.5) then
        call toymd(it,itm)
        jd=juldat(itm)
      endif
      if(nt.eq.6) jd=juldat(it)
      finish=jd+df
c
c get magnitude limit
c
      call getone('magn',bmag,nf)
c
c get input filename, and read the data
c
      call getchr('inpu',name,iok)
      if (iok.eq.0) then
        write(6,100)
 100    format(' Input file name needed ')
        stop
      endif
      open(lui,file=name,status='unknown')
 5    read(lui,101,end=7) inline
 101  format(a80)
      do ichar=1,80
        if (inline(ichar:ichar).eq.'/') inline(ichar:ichar)=' '
        if (inline(ichar:ichar).eq.':') inline(ichar:ichar)=' '
      enddo
      read(inline,*,err=5) (it(i),i=1,5),sec,tlat,tlong,tdep,tmag
      it(6)=nint(sec)
      df=(it(4)+(it(5)+it(6)/60.)/60.)/24.
      jd=juldat(it)
      ttime=jd+df
c apparently GE 5.0.1 cannot handle longitudes above 180
c      if(tlong.lt.0) tlong=tlong+360.
      if(tlong.gt.180) tlong=tlong-360.
      if(ttime.ge.start.and.ttime.le.finish.and.
     1 tlat.ge.geol(1).and.tlat.le.geol(2).and.
     2 tlong.ge.geol(3).and.tlong.le.geol(4).and.
     3 tmag.ge.bmag) then
         rlat(neq)=tlat
         rlong(neq)=tlong
         dep(neq)=tdep
         time(neq)=ttime
         rmag(neq)=tmag
         neq=neq+1
      endif
      if(neq.le.maxeq) go to 5
      write(6,102) maxeq
 102  format(' Only ',i6, ' earthquakes read in')
 7    neq=neq-1
      close(lui)
c
c  get altitude scaling (set to -1 and zero by default)
c
      ascale=1
      zerlev=0
      call getchr('flat',name,iok)
      if(iok.ge.0) then
        ascale=0
      endif
      call getarr('floa',rit,1,nt)
      if(nt.lt.1.and.nt.ge.0) then
        write(6,104)
 104    format(' float needs upwards shift in km ')
        stop
      endif
      if(nt.eq.1) then
        ascale=-1
        zerlev=rit(1)
      endif
      call getarr('alts',rit,2,nt)
      if(nt.lt.2.and.nt.ge.0) then
        write(6,106)
 106    format(' altscale needs scale and upwards shift')
        stop
      endif
      if(nt.eq.2) then
        ascale=-rit(1)
        zerlev=rit(2)
      endif
      return
      end
c=======================================================================
      subroutine dosplit
c
c$$$$ calls getarr,getint,getone,juldat,toymd
c
c  Creates the output file, in KML format, arranging data into folders
c covering different time intervals chosen so that each folder contains
c roughly equal numbers of displayed events.
c
      parameter (maxeq=150000)
      parameter (maxspl=50)
      double precision start,finish,time,split,splpts,vanish,
     1  sort1,sort2,tt
      dimension sort1(maxeq),sort2(2*maxeq),isum(2*maxeq),key(2*maxeq),
     1  rit(6),it(6),itm(3),fpar(2),tt(2)
c
c common block for earthquake information
c
      common /eqs/rlat(maxeq),rlong(maxeq),dep(maxeq),time(maxeq),
     1            rmag(maxeq),neq
c
c common block for display information
c
      common /display/vanish(maxeq),splpts(maxspl),start,finish,
     1 fade,nsplit,nspp
c
c
c  Get number of folders, parameters for fading
c
      call getint('nspl',nsplit,nf)
      nsplit=nsplit-1
      call getarr('fade',fpar,2,nf)
      fade=fadeget(fpar(1),fpar(2))
c
c  Find the end times of the icons associated with each earthquake
c
      do i=1,neq
        call fadeout(fade,rmag(i),0,0,time(i),tt)
        vanish(i)=tt(1)
      enddo
c
c  To find the times that put an equal number of icons into each
c    folder, we need a sorted list of start and end (vanishing)
c    times, with a pointer to the sums of start times and end times.
c
c  The times are probably read in as sorted, but we sort them anyway
c
      do i=1,neq
        sort1(i)=time(i)
      enddo
      call shells(sort1,key,neq)
      do i=1,neq
        sort2(2*i-1)=sort1(i)
        sort2(2*i)=vanish(i)
        if(vanish(i).gt.finish) sort2(2*i)=finish
        isum(2*i-1)=key(i)
        isum(2*i)=-key(i)
      enddo
c
c Now sort the complete set of on and off times; isum(key(i) is the
c  matching array of sums
c
      call shells(sort2,key,2*neq)
      splpts(1)=start
      splpts(nsplit+1)=finish
c
c   As an adequate approximation, put divisions at times that include
c  equal numbers of events, not intervals
c
      nperf=neq/nsplit
      do i=2,nsplit
      splpts(i)=sort1(nperf*(i-1))
C      write(6,*) splpts(i-1),splpts(i),nperf
      enddo
c
c   Loop for finding split points (values of  start  and   finish provide
c the other two boundaries needed). At each step, the time is set to
c include equal numbers of icons by including all the "new" events, but
c then backing up by the number that cross the bottom boundary
c
c  Check for possible split time set by user, and insert it if given
c
      nspp=0
      call getarr('spli',rit,6,nt)
      do i=1,nt
        it(i)=rit(i)
      enddo
      if(nt.ge.5) then
        df=(it(nt-2)+(it(nt-1)+it(nt)/60.)/60.)/24.
        if(nt.eq.5) then
          call toymd(it,itm)
          jd=juldat(itm)
        endif
        if(nt.eq.6) jd=juldat(it)
        split=jd+df
        do i=nsplit+1,1,-1
          if(splpts(i).gt.split) splpts(i+1)=splpts(i)
        enddo
        nsplit=nsplit+1
        do i=1,nsplit-1
          if(splpts(i).eq.splpts(i+1)) then
            splpts(i)=split
            nspp=i
          endif
        enddo
      endif
      return
      end
c=======================================================================
      subroutine kmlmak
c
c  Write out the information about the earthquakes in KML format
c
      parameter (maxeq=150000)
      parameter (maxspl=50)
      character*80 name
      double precision start,finish,time,splpts,t1,t2,tt,vanish,sp1,sp2
      dimension it1(6),it2(6),tt(2)
c
c common block for earthquake information
c
      common /eqs/rlat(maxeq),rlong(maxeq),dep(maxeq),time(maxeq),
     1            rmag(maxeq),neq
c
c common block for display information
c
      common /display/vanish(maxeq),splpts(maxspl),start,finish,
     1 fade,nsplit,nspp
      data luo/12/
c
c Get output filename and open file
c
      call getchr('outp',name,iok)
      if (iok.eq.0) name(1:7)='doc.kml'
      open(luo,file=name)
c
c Write the header
c
      call ghed(luo)
c
c Loop through each time segment (folder) and write out the KML instructions for
c   each earthquake--which may involve multiple icons. First decide if
c   earthquake appears at all: if so, run through all possible fade values,
c   writing an entry only if the times fall within the segment
c
      do i=1,nsplit
        t1=splpts(i)
        t2=splpts(i+1)
        sp1=t2-t1
        call jul2dat(t1,it1)
        call jul2dat(t2,it2)
c
c Write folder information
c
        write(luo,110)
 110    format('<Folder>')
        if(nspp.eq.0) write(luo,112) it1,it2
 112    format('  <name>From ',i4,' ',i2.2,' ',i2.2,'  ',
     1                        i2.2,':',i2.2,':',i2.2,
     2  ' to ',i4,' ',i2.2,' ',i2.2,'  ',
     3       i2.2,':',i2.2,':',i2.2,'</name>')
        if(nspp.ne.0) write(luo,114) t1-splpts(nspp),t2-splpts(nspp)
 114    format('  <name>From ',f8.3,' to ',f8.3,' days </name>')
c
c Loop over all earthquakes; for each one, check if it would be visible
c  in this folder
c
        do j=1,neq
          if(time(j).le.t2.and.vanish(j).ge.t1) then
            sp2=vanish(j)-time(j)
            r=sp2/sp1
c
c Number of fade levels depends on ratio of total visibility time to span
c  covered by folder.
            if(r.lt.0.02) then
              call fadeout(fade,rmag(j),10,1,time(j),tt)
              if(tt(1).le.t2.and.tt(2).ge.t1) then
                tt(1) = max(tt(1),t1)
                tt(2) = min(tt(2),t2)
                call writeout(luo,tt,time(j),rlat(j),rlong(j),dep(j),
     1                        rmag(j),10)
              endif
            endif
            if(r.gt.0.02.and.r.lt.0.125) then
               do k=10,1,-3
c
c Loop over some levels of fading; for each one, see if the time interval
c   falls within the folder interval, and trim it to do so if need be.
c   Then write out the information using subroutine writeout
c
                 call fadeout(fade,rmag(j),k,k,time(j),tt)
                 if(tt(1).le.t2.and.tt(2).ge.t1) then
                   tt(1) = max(tt(1),t1)
                   tt(2) = min(tt(2),t2)
                   call writeout(luo,tt,time(j),rlat(j),rlong(j),dep(j),
     1                        rmag(j),k)
                 endif
               enddo 
            endif
            if(r.gt.0.125) then
               do k=10,1,-1
c
c As above, but for all levels
c
                 call fadeout(fade,rmag(j),k,k,time(j),tt)
                 if(tt(1).le.t2.and.tt(2).ge.t1) then
                   tt(1) = max(tt(1),t1)
                   tt(2) = min(tt(2),t2)
                   call writeout(luo,tt,time(j),rlat(j),rlong(j),dep(j),
     1                        rmag(j),k)
                 endif
               enddo 
            endif
          endif
c end earthquake loop
        enddo
        write(luo,120)
 120    format('</Folder>')
      enddo
      write(luo,130)
 130  format('</Document>',/,'</kml>')
      return
      end
c=======================================================================
      subroutine ghed(lu)
c
c$$$$ calls getchr
c
c  Writes out the header information for the KML file, primarily the icon
c   names used internally and how they match to the icons in the /images
c   directory. The icon names contain a letter for the transparency level,
c   and a number for the magnitude: e.g., A4.50 or K6.00
c
      parameter (maxeq=150000)
      parameter (maxspl=50)
      character*80 name
      character*1 tcod
      character*2 ncod
      character*3 sh
      character*10 col
      double precision start,finish,splpts,vanish
      dimension tcod(11),ncod(11),s(4),col(4),sh(3)
c
c common block for display information
c
      common /display/vanish(maxeq),splpts(maxspl),start,finish,
     1 fade,nsplit,nspp
c
c  tcod  is letters for different icon transparencies, and  ncod
c   is the corresponding strings (which are the percentages) used in
c   the actual icon names
c
      data tcod/'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K'/
      data ncod/'01','10','20','30','40','50','60','70','80','90','99'/
      data sh/'dot','sqr','tri'/
      data col/'.ff.ff.ff.','.00.00.00.','.ff.00.00.','.ff.00.ff.'/
      data name(1:3)/'xxx'/
c
c default values for icon size: first two are scale factors corresponding
c  to the limiting magnitudes (the second two)
c
      data s/1.5,0.05,9.5,0.5/
      data ic/1/,is/1/
c
c  Get color and shape parameters
c
      call getchr('color',name,len)
      if(name(1:3).eq."whi") ic=1
      if(name(1:3).eq."bla") ic=2
      if(name(1:3).eq."red") ic=3
      if(name(1:3).eq."mag") ic=4
      call getchr('shape',name,len)
      if(name(1:3).eq."cir") is=1
      if(name(1:3).eq."dot") is=1
      if(name(1:3).eq."squ") is=2
      if(name(1:3).eq."tri") is=3
      write(lu,100)
 100  format(
     1  '<?xml version="1.0" encoding="UTF-8"?>',/,
     2  '<kml xmlns="http://earth.google.com/kml/2.1">',/,
     3  '<Document>')
      call getchr('name',name,len)
      if(len.gt.0) write(lu,102) name(1:len)
 102  format('  <name>',a,'</name>')
c
c  Get size parameters, and loop over all magnitudes (times 10),
c   which set the icon size
c
      call getarr('size',s,4,nf)
      mu=nint(10*s(3))
      ml=nint(10*s(4))
      do m=ml,mu
         sl=(s(1)-s(2))/(s(3)-s(4))
         sc=sl*((m/10.)-s(4))+s(2)
C        sl=(1.5-0.05)/(9.5-0.5)
C        sc=sl*((m/10.)-0.5)+0.05
c
c  Loop over all 11 transparency levels
c
        do i=1,11
          if(sc.gt.0) then
             write(lu,110) tcod(i),m,sc/2
 110         format(
     1   '   <Style id="',a1,i2.2,'">',/,
     2   '   <BalloonStyle>',/,
     3   '     <text><![CDATA[',/,
     4   '        $[description]<br/>',/,
     5   '     ]]></text>',/,
     6   '   </BalloonStyle>',/,
     7   '     <IconStyle>',/,
     8   '       <scale>',f4.2,'</scale>')
              write(lu,112) sh(is),col(ic),ncod(i)
 112         format(
     1   '       <Icon><href>images/',a3,a10,a2,'.png</href></Icon>',/,
     3   '     </IconStyle>',/,
     4   '   </Style>')
          endif
        enddo
      enddo
      return
      end
c=======================================================================
      subroutine writeout(lu,tt,time,rlat,rlong,dep,rmag,k)
c
c$$$$ calls jul2dat
c
c  Writes out a Placemark element for an earthquake with given fade level;
c   tt is the start and end time for the earthquake at that fade level (set
c   by k, which runs from 10 (opaque) to 0 (just visible)
c
      double precision tt,time
      character*44 f1
      character*19 f2
      character*16 f3
      character*31 f4
      character*9 latf,longf
      character*2 htf
      character*130 pform
      character*3 month
      character*1 tcod
      dimension month(12)
      dimension tcod(11)
      dimension tt(*)
      dimension it1(6),it2(6)
c
c common block for scaling depth to elevation
c
      common /depscl/ascale,zerlev
      data f1/'(''<Point>'',/,''<altitudeMode>relativeToGround'/
      data f2/'</altitudeMode>'',/,'/
      data f3/'''<coordinates>'','/
      data f4/',''</coordinates>'',/,''</Point>'')'/
      data month/'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug',
     1            'Sep','Oct','Nov','Dec'/
c
c  tcod  is letters for different icon transparencies: A is 1%, K 99%
c
      data tcod/'A','B','C','D','E','F','G','H','I','J','K'/
      call jul2dat(tt(1),it1)
      call jul2dat(tt(2),it2)
c
c   Find altitude of earthquake; if less than zero, do not write it out
c
      alt=ascale*dep + zerlev
      if(alt.lt.0) return
c
c   Find size part of icon identifier
c
      if(rmag.lt.0.5) rmag=0.5
      mag=10*rmag
      write(lu,100) tcod(k+1),mag
 100  format('<Placemark>',/,' <styleUrl>#',a1,i2.2,'</styleUrl>')
      write(lu,102) it1
 102  format('<TimeSpan>',/,'  <begin>',i4,'-',i2.2,'-',i2.2,'T',
     1               i2.2,':',i2.2,':',i2.2,'Z','</begin>')
      write(lu,104) it2
 104  format('  <end>',i4,'-',i2.2,'-',i2.2,'T',
     1          i2.2,':',i2.2,':',i2.2,'Z','</end>',/,'</TimeSpan>')
c
c  Rather elaborate object-time formatting is to avoid blanks in coordinates
c
      id=int(1000*alt)
      if(rlat.ge.10) latf='f7.4,'','','
      if(rlat.ge.0.and.rlat.lt.10) latf='f6.4,'','','
      if(rlat.lt.0.and.rlat.gt.-10) latf='f7.4,'','','
      if(rlat.le.-10) latf='f8.4,'','','
      if(rlong.ge.0.and.rlong.lt.10) longf='f6.4,'','','
      if(rlong.ge.10.and.rlong.lt.100) longf='f7.4,'','','
      if(rlong.ge.100) longf='f8.4,'','','
      if(rlong.lt.0.and.rlong.gt.-10) longf='f7.4,'','','
      if(rlong.lt.-10.and.rlong.lt.100) longf='f8.4,'','','
      if(rlong.lt.-100) longf='f9.4,'','','
      if(alt.lt.10) htf='i4'
      if(alt.ge.10.and.dep.lt.100) htf='i5'
      if(alt.ge.100) htf='i6'
      pform=f1//f2//f3//longf//latf//htf//f4
      write(lu,pform) rlong,rlat,id
c
c  Description is the time and magnitude
c
      call jul2dat(time,it1)
      write(lu,112) it1(3),month(it1(2)),it1(1),it1(4),it1(5),
     1   rmag,rlat,rlong,dep
 112  format('<description>',/,'<![CDATA[',/,i6,a4,i5,
     1  '(',i2,':',i2,'); Mag ',f4.2,3x,f9.4,'N',2x,f10.4,'W',
     2   4x,f5.1,' km',/,']]>',/,'</description>')
      write(lu,120)
 120  format('<Snippet maxLines="0"></Snippet>',/,'</Placemark>')
      return
      end
c=======================================================================
      function fadeget(time,rmag)
c
c$$$$ calls no other routines
c
c Function to determine time constant for fadeout; obtained from time
c   taken for an earthquake of magnitude  rmag  to disappear, which is
c   assumed to be 19*tau (the value of t where the fade parameter
c   becomes .05)
      x=exp(3.45*rmag)
c   time=19*fade*e(rm)
      fadeget = time/(19*x)
      return
      end
c=======================================================================
      subroutine fadeout(a,rm,l1,l2,time,tt)
c
c$$$$ calls no other routines
c
c Subroutine to determine times (given in tt) at which an earthquake of
c   magnitude rm will start at level l1 and end at level l2. Levels run
c   from 10 to 1 (0 is fully transparent). Variable  time  is the
c   origin time of the earthquake, and a the fadeout parameter.
c
      double precision time,tt
      dimension tt(*),r(2)
      if(l1.gt.10) l1=10
c
c Set levels to be multiples of 0.1, +- 0.05
c
      r(1) = l1/10.+0.05
      r(2) = l2/10.-0.05
c
c Nondimensional times are set so if l1=10, tt(1)=time; if l2=0, tt(2)=time
c
      tau=a*exp(3.45*rm)
      do i=1,2
        tt(i)=time+tau*amax1(0.,(1-r(i))/r(i))
      enddo
      return
      end
c=======================================================================
      subroutine jul2dat(t,it)
c
c$$$$ calls datjul
c
c Converts from Julian day and fraction of day (0.5 less than actual
c  Julian date) to yr-mon-day-hr-min-sec
c
      double precision t,f,h
      dimension it(*)
      jd=int(t)
      call datjul(jd,it)
      f=t-jd
      h=24*f
      it(4)=int(h)
      f=h-it(4)
      h=60*f
      it(5)=int(h)
      f=h-it(5)
      h=60*f
      it(6)=nint(h)
      return
      end
c=======================================================================
      subroutine datjul(jd,it)
c
c$$$$ calls no other routines
c
c  Converts Julian Date jd into year-month-day (Gregorian calendar) in
c    array it. Algorithm from p. 6??, Explanatory Supplement Amer Ephemeris
c    & Nautical Almanac
c
      dimension it(*)
      k = jd+68569
      n = 4*k/146097
      k = k - (146097*n+3)/4
      it(1) = 4000*(k+1)/1461001
      k = k - 1461*it(1)/4 + 31
      it(2) = 80*k/2447
      it(3) = k - 2447*it(2)/80
      k = it(2)/11
      it(2) = it(2) + 2 - 12*k
      it(1) = it(1) + 100*(n-49) + k
      return
      end
c=======================================================================
      subroutine toymd(it1,it2)
      dimension it1(*),it2(*)
c
c$$$$ calls no other routines
c
c  Converts times in it1 given as year and day to year-month-day in it2
c
      idn(m) = mod((367*(m-2-12*((m-14)/12)))/12+29,365)
      mon(jj,m) = (12*(jj-29-m))/367 + 2 + (jj-200)/169
      it2(1)=it1(1)
      it2(2)=mon(it1(2),leap(it1(1)))
      it2(3) = it1(2) - idn(it2(2)) - leap(it2(1))*((9+it2(2))/12)
      return
      end
c=======================================================================
      function juldat(it)
      dimension it(*)
c
c$$$$ calls no other routines
c
c  Finds Julian Date from Gregorian date, Algorithm from p. 604, Explanatory
c    Supplement Amer Ephemeris & Nautical Almanac (cf Comm CACM, 11, 657 (1968)
c    and 15, 918 (1972)) Valid for all positive values of Julian Date
c
      juldat=(1461*(it(1)+4800+(it(2)-14)/12))/4
     1     + (367*(it(2)-2-12*((it(2)-14)/12)))/12
     2     - (3*((it(1)+4900+(it(2)-14)/12)/100))/4+it(3)-32075
      return
      end
c=======================================================================
      function leap(iy)
c
c$$$$ calls no other routines
c
c returns 1 if year is a leap year, by Clavian (Gregorian) rule
c
      leap = 1 - (mod(iy,4)+3)/4
      if(mod(iy,100).eq.0.and.mod(iy,400).ne.0) leap=0
      return
      end
c=======================================================================
      subroutine shells(x,k,n)
c  sorts an array x, of length n, sorting upward, and returns
c  an array k which may be used to key another array to the
c  sorted pattern (i.e., if we had an array f to which x 
c  corresponded before sorting, then after calling shells,
c  f(k(1)) will be the element of f corresponding to the
c  smallest x, f(k(2)) the next smallest, and so on.
c   revised 29-dec-82 so k is sorted in turn, the values of
c  k that point to identical values of x being put in increasing
c  order
c
c$$$$ calls no other routines
c
      double precision x,sv
      dimension x(*),k(*)
      igap = n
      do i = 1,n
         k(i) = i
      enddo
 5    if(igap.le.1) go to 25
      igap = igap/2
      imax = n - igap
 10   iex = 0
      do i = 1,imax
         ipl = i + igap
         if(x(i).gt.x(ipl)) then
            sv = x(i)
            ik = k(i)
            x(i) = x(ipl)
            k(i) = k(ipl)
            x(ipl) = sv
            k(ipl) = ik
            iex = iex + 1
         endif
      enddo
      if(iex.gt.0) go to 10
      go to 5
c
c  now sort k's (for identical values of x, if any)
c
 25   j = 1
 30   if(j.ge.n) return
      if(x(j).eq.x(j+1)) go to 33
      j = j + 1
      go to 30
c  have at least two x's with the same value - see how long this is true
 33   l = j
 35   if(x(l).ne.x(l+1)) go to 38
      l = l + 1
      if(l.lt.n) go to 35
c  j and l are the indices within which x(i) does not change - sort k
 38   igap = l - j + 1
 40   if(igap.le.1) j = l + 1
      if(igap.le.1) go to 30
      igap = igap/2
      imax = l-j+1 - igap
 45   iex = 0
      do i=1,imax
      ipl = i + igap + j - 1
      if(k(i+j-1).gt.k(ipl)) then
         ik = k(i+j-1)
         k(i+j-1) = k(ipl)
         k(ipl) = ik
         iex = iex + 1
         endif
      enddo
      if(iex.gt.0) go to 45
      go to 40
      end

c=======================================================================
c ***********************************************************************
c *                                                                     *
c *   Routines below for reading and processing script commands are     *
c *     provided courtesy of R. L. Parker; specific commands in review  *
c *     listing in scanrp, and checking in icheck have been modified    *
c *     for UDQ code.                                                   *
c *                                                                     *
c ***********************************************************************
c=======================================================================
      subroutine scanrp
c
c$$$$ calls clear getchr icheck
c
c  Input routine for commands 
c
c  Reads from the standard input until eof or code "execute " or "quit".
c  Saves lines in the input store  /store/ for later retrieval by
c  getarr, getone or getchr.
c
c  Prints a glossary upon request
c
      parameter (inmx=500)
      character*4 code
      character*80 input(inmx),line
      common /store/ input
      common /ndict/ iecho,nin
c
      if (nin .eq. 0) write(6,'(a)') ' ',
     $'Enter commands for producing UDQ file (? for help)'
c
      do 1500 l=nin+1, inmx
        read(*,'(80a)', end=3000) line
        if (line(1:4).eq.'cont' .or. line(1:4).eq.'exec') goto 2000
c  List a glossary of codes
        if (line(1:1) .eq. '?') then
          write(6, '(a/a/(2x,a))')
     $'Enter commands from the following list:',
     $'?:        Remind me of the command list again',
     $'execute:           Quit reading commands and begin calculations',
     $'input filename:   Enter filename of seismicity data (mandatory)',
     $'output filename:                    Enter filename for KML file',
     $'area south north west east:          Give lat and lon to select',
     $'magnitude minmag:             Give minimum magnitude to include',
     $'name  namestring                     Give name to be used by GE',
     $'start yr mo day hr min sec:        Enter start time to be taken',
     $'                (this may also be given as   yr doy hr min sec)',
     $'finish yr mo day hr min sec:         Enter end time to be taken',
     $'                (this may also be given as   yr doy hr min sec)',
     $'split yr mo day hr min sec:         Enter time for forced split',
     $'                (this may also be given as   yr doy hr min sec)',
     $'nsplit n:      Enter number of time segments to split data into',
     $'fadeout time magnitude:          Enter fadeout time (d) for mag' 

          write(6, '(2x,a)')
     $'flatten:                              Put all quakes on surface',
     $'float a:                 Plot quakes right-side-up, lifted by a',
     $'altscale a b:                     Plot quakes at altitude a*d+b',
     $'size r:                                   Enter size parameters',
     $'color cname:      Enter color, cname is white|red|black|magenta',
     $'shape sname:          Enter shape, sname is dot|square|triangle',
     $'review:                           Display current command stack',
     $'clear command:     Delete most recent occurrence of the command',
     $' '
c
c  Review the command stack
        elseif (line(1:4) .eq. 'revi') then
          write(6,'(5x,a)')' ', '=================== ',
     $    (input(i)(1:60),i=1,nin),'=================== '
c
        elseif (line(1:1) .ne. ' ') then
          if (line(1:4) .eq. 'echo') iecho=-iecho
          if (line(1:4) .eq. 'quit') then
            write(6,'(/a)') 'Normal termination'
            stop
          endif
          nin=nin + 1
          if (nin .gt. inmx) then
            write(6,'(a)') '>>>> Too many commands - memory full'
            stop
          endif
          call icheck(line)
          input(nin)=line
c  Clear a command and clear the clear command
          if (line(1:4) .eq. 'clea') then
            call getchr('clear', code, klear)
            call clear (code)
            nin=nin - 1
          endif
        endif
 1500 continue
c
 2000 continue
      n1=max(1, nin-24)
      write(6,'(5x,a)')' ', '=================== ',
     $(input(i)(1:60),i=n1,nin),'=================== '
      return
c
 3000 stop
      end
c=======================================================================
      subroutine icheck(line)
c
c$$$$ calls no other routines
c
c  Checks the command fragment com against a list; appends a
c  warning if com is not present in the list.
      character*80 line
      character*4 com
      com=line(1:4)
      if (0 .eq.
     1 index('area clea exec fade fini inpu magn nspl outp',com)
     2+index('name revi size spli star colo shap flat floa',com)
     3+index('alts',com))
     4 line=line(1:20)//' <<<<<<< Unrecognized command'
      return
      end
c=======================================================================
      subroutine getarr(code, values, nwant, nfound)
c
c$$$$ calls ljust
c
c
c  Extracts an array of numbers from the input store.  That store is
c  a large array in common /store/ which has been filled earlier.
c
c  code    A 4-bye identifying code.  Only lines in the input store
c          beginning with this code are scanned for numbers.
c  values  the real output array of values found.
c  nwant   the maximum number of numbers expected .
c  nfound  the number of numbers actually found in the input.
c          If the line contains fewer than nwant  values, this is the
c          value returned in  nfound.  If an error is discovered
c          nfound=-n, where  n  is the number of numbers properly
c          decoded.  If there are no numbers after the codeword
c          nfound=0.  Finally, if the code is absent from the store
c          nfound=-99 and the array is left undisturbed.
c
      parameter (inmx=500)
      character *80 input(inmx),line,local,char, code*4
      common /store/ input
      common /ndict/ iecho,nin
      dimension values(*)
c
c  Read the store in reverse order (Thus last entry is obeyed)
ce
      do 1010 lin=nin, 1, -1
        line=input(lin)
c  Check for code 
        if (code .eq. line(1:4)) goto 1020
 1010 continue
c  Code word not found
      nfound=-99
      return
c
 1020 if (iecho .ge. 1) write(6,'(2a)')'==> ',line
      n1=index(line, ' ')+1
      n2=index(line, '%')
      n2=80 + min(n2,1)*(n2 - 81)
      char=line(n1:n2)
c
      do 1500 n=1, nwant
        call ljust(char, local)
        lbl=index(local, ' ')
        if (lbl .eq. 1) goto 1510
        read (local, *, err=2000) values(n) 
        char=local(lbl:80)
 1500 continue
      n=nwant+1
 1510 nfound=n - 1
      return
c
 2000 print '(a)',' ',
     $'>>> Unreadable numbers in this input line:',line
      nfound=1 - n
      return
      end
c=======================================================================
      subroutine getone(code, value, nfound)
c
c$$$$ calls getarr
c
c  Extracts a single number from the input store.  That store is
c  a large array in common /store/ which has been filled earlier.
c
c  code    A 4-bye identifying code.  Only lines in the input store
c          beginning with this code are scanned for numbers.
c  value   the real output variable containing the desired number.
c  nfound  is 1 if a number is successfully read in; it is zero
c          the number is absent or unreadable.  nfound = -99 if the
c          code is absent from the input store.
c
      character*4 code
      dimension v(1)
ce
      call getarr(code, v, 1, nfound)
      if (nfound .eq. 1) value=v(1)
      return
      end
c=======================================================================
      subroutine getint(code, number, nfound)
c
c$$$$ calls getarr
c
c  Extracts a single integer from the input store.  
c  See getone for details.
c
      character*4 code
      dimension v(1)
ce
      call getarr(code, v, 1, nfound)
      if (nfound .eq. 1) then
        number=nint(v(1))
        if (number .ne. v(1)) print '(4a,1p,g16.8/a,i10)',
     $  '>>> Warning: ',code,' expects an integer argument, but',
     $  ' found: ',v(1),'    Returns: ',number,' '
      endif
      return
      end
c=======================================================================
      subroutine getchr(code, char, nbytes)
c
c$$$$ calls ljust
c  Extracts a single character variable from the input store.  That 
c  store is a large array in common /store/ which has been filled 
c  earlier.
c
c  code    A 4-bye identifying code.  Only lines in the input store
c          beginning with this code are scanned.
c  char    the character output variable containing the desired string.
c  nbytes  is the length of the string read; it is zero if
c          the line is blank after the code.  nbytes = -99 if the
c          code is absent from the input store.
c
      parameter (inmx=500)
      character *80 input(inmx),line, char*(*),  code*4
      common /store/ input
      common /ndict/ iecho,nin
c  Inspect the store in reverse order (thus reading latest entry)
ce
      do 1010 lin=nin, 1, -1
        line=input(lin)
c  Check for code word  
        if (code .eq. line(1:4)) goto 1020
 1010 continue
c  Code word not found
      nbytes=-99
      return
c
 1020 if (iecho .ge. 1) write(6,'(2a)')'==> ',line
c  
      n1=index(line, ' ')+1
      n2=index(line, '%')
      n2=80 + min(n2,1)*(n2 - 81)
c  Check for blank string
      nbytes=0
      if (line(n1:n2) .eq. ' ') return
c  Save in char everything from 1st non-blank to last non-blank
      call ljust(line(n1:n2), char)
      nn=min(n2-n1+1, len(char))
      do 1200 k=nn, 1, -1
        if (char(k:k) .ne. ' ') goto 1210
 1200 continue
 1210 nbytes= k
      return
      end
c=======================================================================
      subroutine clear(code)
c
c$$$$ calls no other routines
c
c  Removes the last command entry identified by code.
c  code    A 4-byte identifying code.  Only lines in the input store
c          beginning with this code are scanned.
c
      parameter (inmx=500)
      character *80 input(inmx),line, code*4
      common /store/ input
      common /ndict/ iecho,nin
c
c  Inspect the store in normal order
ce
      do 1010 lin= nin, 1, -1
        line=input(lin)
c  Check code  and clear the line if present, but only once
        if (code .eq. line(1:4)) then
          input(lin)=' --'
          return
        endif
 1010 continue
      return
      end
c=======================================================================
      subroutine ljust(str1, str2)
c
c$$$$ calls no other routines
c  Left justify: input character string str1 (up to 80 bytes in length)
c  is left justified, removing leading blanks, and returned in str2.
c  str1 and str2 may be set to the same variable in the call.
      character*(*) str1, str2, str3*80
c
ce
      str2=str1
      if (str1 .eq. ' ') return
c
      l1=len(str1)
      str3=str1
      do 1010 j=1, l1
        if (str1(j:j) .ne. ' ') goto 1011
 1010 continue
 1011 str2=str3(j:l1)
      return
      end
c=======================================================================
      blockdata iounit
      common /ndict/ iecho,nin
c  Don't echo command lines when read
      data iecho/-1/
c  Initial command line count
      data nin/0/
      end
